SystemOrganization addCategory: #FuelBenchmarks!
SystemOrganization addCategory: #'FuelBenchmarks-Tests'!

TestCase subclass: #FLBenchmarksTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Tests'!

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 12/7/2010 10:21'!
benchmarksWithTwoRunnersAndTwoSamples
	
	| benchmarks |
	benchmarks := FLBenchmarks new.
	benchmarks infoWriteStream: '' writeStream.
	benchmarks runners add: FLBenchmarksDummyRunner new.
	benchmarks runners add: FLBenchmarksDummyRunner new.
	benchmarks samples add: (FLBenchmarkSample title: 'sample1' content: #(1 2 3 4)).
	benchmarks samples add: (FLBenchmarkSample title: 'sample2' content: #(5 6 7 8)).
	^ benchmarks! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 12/7/2010 10:14'!
testTwoRunnersTwoSamples
	"Simulating the real runners, each dummy adds two results the the benchmarks"
	
	| benchmarks |
	benchmarks := self benchmarksWithTwoRunnersAndTwoSamples.
	benchmarks run.
	self assert: 8 equals: benchmarks results size! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 12/7/2010 10:36'!
testTwoRunnersTwoSamplesAndFourSubsets
	"Simulating the real runners, each dummy adds two results the the benchmarks. Each sample runs four times"
	
	| benchmarks |
	benchmarks := self benchmarksWithTwoRunnersAndTwoSamples.
	benchmarks numberOfSubsets: 4.
	benchmarks run.
	self assert: 32 equals: benchmarks results size! !

Object subclass: #FLBenchmarkResult
	instanceVariableNames: 'run milliseconds title successful'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
milliseconds

	^ milliseconds! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
milliseconds: anObject

	milliseconds := anObject! !

!FLBenchmarkResult methodsFor: 'printing' stamp: 'MartinDias 12/6/2010 18:49'!
printOn: aStream

	aStream
		nextPutAll: 'sample: ';
		nextPutAll: run sample title;
		nextPutAll: ' size: ';
		nextPutAll: run sample content size printString;
		nextPutAll: ' title: ';
		nextPutAll: title.

	successful 
		ifTrue: [
			aStream 
				nextPutAll: ' ms: ';
				nextPutAll: milliseconds printString ] 
		ifFalse: [
			aStream nextPutAll: ' FAILED' ]
! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:30'!
run
	^ run! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:30'!
run: anObject
	run := anObject! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 11:02'!
sampleSize

	^ run sample content size! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MD 12/3/2010 17:06'!
successful

	^ successful! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MD 12/3/2010 17:07'!
successful: aBoolean

	successful := aBoolean! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
title

	^ title! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
title: anObject

	title := anObject! !

Object subclass: #FLBenchmarkRun
	instanceVariableNames: 'sample runner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarkRun classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/6/2010 17:48'!
sample: aSample runner: aRunner

	^ self basicNew
		initialize;
		sample: aSample;
		runner: aRunner;
		yourself		! !

!FLBenchmarkRun methodsFor: 'running' stamp: 'MartinDias 12/6/2010 18:45'!
runOn: benchmarks

	runner runOn: benchmarks with: sample run: self! !

!FLBenchmarkRun methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 17:49'!
runner
	^ runner! !

!FLBenchmarkRun methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 17:49'!
runner: anObject
	runner := anObject! !

!FLBenchmarkRun methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 17:49'!
sample
	^ sample! !

!FLBenchmarkRun methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 17:49'!
sample: anObject
	sample := anObject! !

Object subclass: #FLBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarkRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 10:40'!
allRunners

	^ self allSubclasses 
		select: [ :aRunnerClass | aRunnerClass shouldRun ] 
		thenCollect: [ :aRunnerClass | aRunnerClass new ]! !

!FLBenchmarkRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 10:39'!
shouldRun

	^ true! !

!FLBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 16:07'!
materialize: anObject on: aStream
	"Hook method for do materializing. The time for that will be included in the benchmarks results."

	self subclassResponsibility! !

!FLBenchmarkRunner methodsFor: 'running' stamp: 'MartinDias 12/6/2010 18:44'!
runOn: benchmarks with: anObject run: aRun

	| aStream |
	aStream := benchmarks streamNamed: self runnerName.
	benchmarks addResultRunning: [ self serialize: anObject on: aStream ] titled: self runnerName, ' Serialization' run: aRun.

	aStream reset.
	benchmarks addResultRunning: [ self materialize: anObject on: aStream ] titled: self runnerName, ' Materialization' run: aRun.
	aStream close.! !

!FLBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 16:08'!
runnerName
	"Hook method for the name"

	self subclassResponsibility! !

!FLBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 16:07'!
serialize: anObject on: aStream
	"Hook method for do serializing. The time for that will be included in the benchmarks results."

	self subclassResponsibility! !

FLBenchmarkRunner subclass: #FLBenchmarksDummyRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Tests'!

!FLBenchmarksDummyRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 10:41'!
shouldRun 
	"Returns false so it's not added to the benchmarks default run'"

	^ false! !

!FLBenchmarksDummyRunner methodsFor: 'running' stamp: 'MartinDias 12/7/2010 10:01'!
runOn: benchmarks with: anObject run: aRun

	benchmarks addResultRunning: [ ] titled: 'Serialization' run: aRun.
	benchmarks addResultRunning: [ ] titled: 'Materialization' run: aRun.
! !

FLBenchmarkRunner subclass: #FLFuelBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:35'!
materialize: anObject on: aStream

	(FLMaterializer on: aStream) materialize! !

!FLFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:36'!
runnerName

	^ 'Fuel'! !

!FLFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:35'!
serialize: anObject on: aStream

	(FLSerializer on: aStream) serialize: anObject! !

FLBenchmarkRunner subclass: #FLReferenceStreamBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLReferenceStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:42'!
materialize: anObject on: aStream

	(ReferenceStream on: aStream) next! !

!FLReferenceStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:37'!
runnerName

	^ 'ReferenceStream'! !

!FLReferenceStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:41'!
serialize: anObject on: aStream

	(ReferenceStream on: aStream) nextPut: anObject ! !

FLBenchmarkRunner subclass: #FLSmartRefStreamBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLSmartRefStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:49'!
materialize: anObject on: aStream

	aStream fileInObjectAndCode! !

!FLSmartRefStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:38'!
runnerName

	^ 'SmartRefStream'! !

!FLSmartRefStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:47'!
serialize: anObject on: aStream

	(SmartRefStream on: aStream) nextPut: anObject! !

Object subclass: #FLBenchmarkSample
	instanceVariableNames: 'content title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/12/2011 10:45'!
allSampleSelectors
		
	^ #(primitiveMethods completeBinaryTree strings symbols smallIntegers floats pairs points rectangles someMethods byteArrays)! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/6/2010 18:10'!
allSamples

	^ self allSampleSelectors 
		collect: [ :aSelector | self fromSelector: aSelector ]! !

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 14:15'!
basicSampleSelectors
		
	^ #(primitiveMethods completeBinaryTree strings smallIntegers floats rectangles)! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2010 14:16'!
basicSamples

	^ self basicSampleSelectors 
		collect: [ :aSelector | self fromSelector: aSelector ]! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MarianoMartinezPeck 5/12/2011 10:44'!
byteArrays
	
	^ (1 to: 1 << 16) 
		collect: [ :index | (index * 123456790) asByteArray  ]! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:04'!
completeBinaryTree 
	| aCollection numberOfLeafs |
	numberOfLeafs := 2 raisedTo: 14.
	aCollection := OrderedCollection new.
	1 to: numberOfLeafs do: [ :i | 
		aCollection add: (
			FLPair new 
				left: (2 * i); 
				right: (2 * i) + 1;
				yourself) ].
	
	[aCollection size > 1] 
		whileTrue: [
			aCollection := aCollection pairsCollect: [ :leftPair :rightPair | 
				FLPair new 
					left: leftPair;
					right: rightPair;
					yourself ] ].

	^ aCollection! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:51'!
floats

	^ (1 to: (2 raisedTo: 16)) collect: [ :i | i * 1.1 ]
! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/6/2010 18:10'!
fromSelector: aSelector

	^ self new 
		title: aSelector asString;
		content: (self perform: aSelector); 
		yourself! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:51'!
imageSegmentWith: anObject

	^ ImageSegment new
		copyFromRoots: (Array with: anObject)
		sizeHint: 1000
		areUnique: true! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:51'!
imageSegmentWithPrimitiveMethods

	^ {self imageSegmentWith: self primitiveMethods}! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:51'!
methods

	^ Smalltalk allClasses 
		inject: OrderedCollection new 
		into: [ :aCollection :aClass | aCollection addAll: aClass methods. aCollection ].
! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:52'!
mooseModelRoot
"
	MooseModel allInstances 
		ifEmpty: [ 
			MooseModel installDefaultModels.
			MooseScripts createModelForPharo11PackagesCore.
	MooseScripts createModelForNetwork.].

	^ MooseModel root.
"! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/7/2010 14:24'!
pairs

	^ (1 to: 1 << 15) 
		collect: [ :i | 
			FLPair new 
				left: i; 
				right: i+1;
				yourself ].
! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/7/2010 14:24'!
points

	^ (1 to: 1 << 15) 
		collect: [ :i | i @ (i+1) ]! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:52'!
primitiveMethods

	^ SystemNavigation default allPrimitiveMethods! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:52'!
rectangles
	
	^ self rectanglesBy: 4! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/7/2010 14:23'!
rectanglesBy: step
	"Returns rectangles sharing more or less integers, depending step is 1, 2, 3 or 4."

	^ (1 to: (2 raisedTo: 14) * step by: step) 
		collect: [ :i | i@(i+1) corner: (i+2)@(i+3) ]
! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/7/2010 14:22'!
smallIntegers
	
	^ (1 to: 1 << 30 by: 1 << 13) asArray! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:52'!
someMethods

	^ (SystemNavigation default allClassesInPackageNamed: 'System') 
		inject: Set new into: [ :aSet :aClass | aSet addAll: aClass methods. aSet ]! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/7/2010 14:21'!
strings
	
	^ (1 to: 1 << 16) 
		collect: [ :index | (index * 123456790) asString ]! !

!FLBenchmarkSample classSide methodsFor: 'contents' stamp: 'MartinDias 12/6/2010 18:53'!
symbols

	^ Symbol allSymbols "last: 1 << 15"! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2010 10:09'!
title: aTitle content: aContent

	^ self new
		title: aTitle;
		content: aContent;
		yourself! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
content
	^ content! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
content: anObject
	content := anObject! !

!FLBenchmarkSample methodsFor: 'operating' stamp: 'MartinDias 12/7/2010 15:01'!
subset: index of: numberOfSubsets
	"Returns a new instance whose content is a subset of self"

	^ self class new
		title: title; 
		content: (content asOrderedCollection first: content size * index // numberOfSubsets);
		yourself! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
title
	^ title! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
title: anObject
	title := anObject! !

Object subclass: #FLBenchmarks
	instanceVariableNames: 'results streamFactoryBlock samples runners infoWriteStream numberOfSubsets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarks commentStamp: 'MartinDias 12/7/2010 14:16' prior: 0!
I implement benchmarks for comparing Fuel with other ways of serializing and materializing.

To run, just run the following codes (the results will be written at Transcript).


FLBenchmarks newBasic run.

or:

FLBenchmarks newFull run.

!

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2010 14:14'!
newBasic

	^ self new 
		samples: (FLBenchmarkSample basicSamples);
		runners: (FLBenchmarkRunner allRunners);
		useMemoryStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2010 14:12'!
newFull

	^ self new 
		samples: (FLBenchmarkSample allSamples);
		runners: (FLBenchmarkRunner allRunners);
		useMemoryStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 5/11/2011 17:31'!
newFullFuelOnly

	^ self new 
		samples: (FLBenchmarkSample allSamples);
		runners: (Array with: FLFuelBenchmarkRunner new);
		useMemoryStream;
		yourself.! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MarianoMartinezPeck 5/12/2011 10:05'!
addResultRunning: aBlockClosure titled: aTitle run: aRun

	| timeToRun successful |

	"In CogVM the first time a method is used, it is executed in the old slow way, that is, interpreted, and it is added to the cache.. The second time a method is used, that means, when it is found in the cache, Cog converts that method to machine code. However, to do that, extra time is needed. Only the third time the method will be executed as machine code and without extra effort. So, for the samples we run 2 times the same sample so that to be sure to be always in the same condition."
	2 timesRepeat: [aBlockClosure value].
	"A GC can occur at any time and we cannot control it. However, doing a GC before the probe, we minimize the chances of having a GC during the probe and put noise in the measure because of the GC time."
	3 timesRepeat: [Smalltalk garbageCollect].    

	[ timeToRun := aBlockClosure timeToRun. successful := true ]
		on: Error
		do: [ timeToRun := 0. successful := false ].

	results add: (FLBenchmarkResult new
		title: aTitle;
		run: aRun;
		milliseconds: timeToRun;
		successful: successful;
		yourself)! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
infoWriteStream
	^ infoWriteStream! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
infoWriteStream: anObject
	infoWriteStream := anObject! !

!FLBenchmarks methodsFor: 'initialize-release' stamp: 'MartinDias 12/6/2010 19:20'!
initialize 

	super initialize.
	infoWriteStream := Transcript.
	numberOfSubsets := 1.
	results := OrderedCollection new.
	runners := OrderedCollection new.
	samples := OrderedCollection new.! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 19:20'!
numberOfSubsets
	^ numberOfSubsets! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 19:20'!
numberOfSubsets: anObject
	numberOfSubsets := anObject! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 12/3/2010 00:27'!
printOn: aStream

	results do: [ :aResult | 
		aResult printOn: aStream.
		aStream cr ].
	aStream cr! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 00:39'!
results

	^ results! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
results: anObject
	results := anObject! !

!FLBenchmarks methodsFor: 'running' stamp: 'MartinDias 12/6/2010 18:54'!
run

	self runs do: [ :aRun |
		Smalltalk garbageCollect.
		aRun runOn: self.
	] displayingProgress: 'Running benchmarks'.

	self printOn: infoWriteStream.
	infoWriteStream flush.
! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:19'!
runners

	^ runners! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
runners: anObject
	runners := anObject! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 19:01'!
runs

	| runs |
	runs := OrderedCollection new.
	self samples do: [ :aSample | 
		self runners do: [ :aRunner |
			runs addAll: (self runsForSample: aSample runner: aRunner) ] ].
	^ runs.

! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 09:48'!
runsForSample: aSample runner: aRunner

	^ (1 to: numberOfSubsets) collect: [ :index |
		FLBenchmarkRun 
			sample: (aSample subset: index of: numberOfSubsets)
			runner: aRunner ]! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
samples
	^ samples! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
samples: anObject
	samples := anObject! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
streamFactoryBlock
	^ streamFactoryBlock! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
streamFactoryBlock: anObject
	streamFactoryBlock := anObject! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MartinDias 12/3/2010 00:19'!
streamNamed: aName

	^ streamFactoryBlock value: aName! !

!FLBenchmarks methodsFor: 'initialize-release' stamp: 'MD 12/3/2010 16:47'!
useFileStream

	streamFactoryBlock := [ :aName | (StandardFileStream forceNewFileNamed: 'BenchFor', aName) binary ]! !

!FLBenchmarks methodsFor: 'initialize-release' stamp: 'MartinDias 12/3/2010 00:16'!
useMemoryStream

	streamFactoryBlock := [ :aName | (RWBinaryOrTextStream on: '') binary ]! !

Object subclass: #FLBenchmarksSubsetsCSVExporter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarksSubsetsCSVExporter commentStamp: '<historical>' prior: 0!
My goal is export benchmark results to a CSV file. Useful for creating a chart with tools like OpenOffice or Excel.

An example of use:

FLBenchmarksSubsetsCSVExporter 
	export:
		(FLBenchmarks new
			samples: { FLBenchmarkSample fromSelector: #someMethods }; 
			numberOfSubsets: 8;
			runners: (FLBenchmarkRunner allRunners);
			useMemoryStream;
			run;
			yourself) 
	onFileNamed: 
		'someMethods.csv'
!

!FLBenchmarksSubsetsCSVExporter classSide methodsFor: 'exporting' stamp: 'MartinDias 12/6/2010 17:09'!
export: benchmarks onFileNamed: aFileName

	| aStream |
	aStream := FileStream forceNewFileNamed: aFileName.
	[ (self on: aStream)	nextPutResults: benchmarks results ] ensure: [ aStream close ].
	
	Transcript 
		nextPutAll: 'Results exported to ';
		nextPutAll: aFileName;
		flush.! !

!FLBenchmarksSubsetsCSVExporter classSide methodsFor: 'instance creation' stamp: 'MartinDias 11/11/2010 18:01'!
on: aStream

	^ self basicNew 
		initialize;
		stream: aStream;
		yourself! !

!FLBenchmarksSubsetsCSVExporter methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 10:47'!
nextPutColumnNameRowOf: aBenchmarkResults
	"Writes the column names row"

	stream nextPutAll: 'Sample size'.
	(SortedCollection sortBlock: [ :x :y | (x title) <= (y title) ])
		addAll: aBenchmarkResults anyOne;
		do: [ :aResult |
			stream nextPutAll: ', ', aResult title ].
	stream cr.! !

!FLBenchmarksSubsetsCSVExporter methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 10:38'!
nextPutDataRowsOf: aBenchmarkResults
	"Writes the data rows"
		
	aBenchmarkResults do: [ :aResultCollection | 
		"Sample size"
		stream nextPutAll: aResultCollection anyOne sampleSize printString.
	
		"Milliseconds"
		(SortedCollection sortBlock: [ :x :y | (x title) <= (y title) ])
			addAll: aResultCollection; 
			do: [ :aResult | 
				stream nextPutAll: ', ', aResult milliseconds printString
		].
		stream cr.
	].
! !

!FLBenchmarksSubsetsCSVExporter methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 12:02'!
nextPutResults: aBenchmarkResults

	| resultsGroupedBySampleSize |
	
	resultsGroupedBySampleSize := 
		(SortedCollection sortBlock: [ :x :y | (x anyOne sampleSize) <= (y anyOne sampleSize) ])
			addAll: (aBenchmarkResults groupedBy: [ :x | x sampleSize ]);
			yourself.
	
	self nextPutColumnNameRowOf: resultsGroupedBySampleSize.
	self nextPutDataRowsOf: resultsGroupedBySampleSize.! !

!FLBenchmarksSubsetsCSVExporter methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 10:36'!
stream

	^ stream! !

!FLBenchmarksSubsetsCSVExporter methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 10:36'!
stream: anObject

	stream := anObject! !
